"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StakingClient = void 0;
const web3_js_1 = require("@solana/web3.js");
const bn_js_1 = __importDefault(require("bn.js"));
class StakingClient {
    constructor(ctx, pda) {
        this.ctx = ctx;
        this.pda = pda;
    }
    stake(amount) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const tx = new web3_js_1.Transaction();
            const flexibleAccount = this.pda.flexibleStakingAccount(this.ctx.wallet.publicKey);
            const flexibleAccountInfo = yield this.ctx.connection.getAccountInfo(flexibleAccount.key);
            if (!flexibleAccountInfo) {
                const initFlexibleAccountIx = yield this.ctx.methods.initFlexibleAccount({
                    accounts: {
                        user: this.ctx.wallet.publicKey,
                    },
                    inputs: {
                        bump: flexibleAccount.bump,
                    },
                });
                if (!initFlexibleAccountIx.ix) {
                    throw new Error("Failed to create initializing flexible account instruction");
                }
                tx.add((_a = initFlexibleAccountIx.ix) === null || _a === void 0 ? void 0 : _a.instructions[0]);
            }
            const stakeIx = yield this.ctx.methods.stake({
                accounts: {
                    user: this.ctx.wallet.publicKey,
                },
                inputs: {
                    amount,
                },
            });
            if (!stakeIx.ix) {
                throw new Error("Failed to create staking instruction");
            }
            tx.add((_b = stakeIx.ix) === null || _b === void 0 ? void 0 : _b.instructions[0]);
            return tx;
        });
    }
    withdraw(amount, index) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const tx = new web3_js_1.Transaction();
            const flexibleAccount = this.pda.flexibleStakingAccount(this.ctx.wallet.publicKey);
            const flexibleAccountInfo = yield this.ctx.connection.getAccountInfo(flexibleAccount.key);
            if (!flexibleAccountInfo) {
                throw new Error("Ensure user has staked before withdrawing");
            }
            const withdrawPosition = this.pda.withdrawPosition(this.ctx.wallet.publicKey, new bn_js_1.default(index));
            const withdrawPositionInfo = yield this.ctx.connection.getAccountInfo(withdrawPosition.key);
            if (withdrawPositionInfo) {
                throw new Error("Withdraw position index is already taken");
            }
            const withdrawIx = yield this.ctx.methods.withdraw({
                accounts: {
                    user: this.ctx.wallet.publicKey,
                },
                inputs: {
                    withdrawPositionBump: withdrawPosition.bump,
                    index,
                    amount,
                },
            });
            if (!withdrawIx.ix) {
                throw new Error("Failed to create withdrawing instruction");
            }
            tx.add((_a = withdrawIx.ix) === null || _a === void 0 ? void 0 : _a.instructions[0]);
            return tx;
        });
    }
    withdrawAfterDelay(index) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const tx = new web3_js_1.Transaction();
            const withdrawPosition = this.pda.withdrawPosition(this.ctx.wallet.publicKey, new bn_js_1.default(index));
            const withdrawPositionInfo = yield this.ctx.connection.getAccountInfo(withdrawPosition.key);
            if (!withdrawPositionInfo) {
                throw new Error("Ensure create withdraw position before getting delay amount");
            }
            const withdrawDelayIx = yield this.ctx.methods.withdrawDelay({
                accounts: {
                    user: this.ctx.wallet.publicKey,
                },
                inputs: {
                    index,
                },
            });
            if (!withdrawDelayIx.ix) {
                throw new Error("Failed to create withdrawing after delay instruction");
            }
            tx.add((_a = withdrawDelayIx.ix) === null || _a === void 0 ? void 0 : _a.instructions[0]);
            return tx;
        });
    }
    claim() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const tx = new web3_js_1.Transaction();
            const flexibleAccount = this.pda.flexibleStakingAccount(this.ctx.wallet.publicKey);
            const flexibleAccountInfo = yield this.ctx.connection.getAccountInfo(flexibleAccount.key);
            if (!flexibleAccountInfo) {
                throw new Error("Ensure user has staked before claiming");
            }
            const claimIx = yield this.ctx.methods.claim({
                accounts: {
                    user: this.ctx.wallet.publicKey,
                },
            });
            if (!claimIx.ix) {
                throw new Error("Failed to create claiming instruction");
            }
            tx.add((_a = claimIx.ix) === null || _a === void 0 ? void 0 : _a.instructions[0]);
            return tx;
        });
    }
    lockup(amount, index, vaultIndex) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const tx = new web3_js_1.Transaction();
            const userLockupStakingAccount = this.pda.lockupStakingAccount(this.ctx.wallet.publicKey, new bn_js_1.default(index), new bn_js_1.default(vaultIndex));
            const userLockupStakingAccountInfo = yield this.ctx.connection.getAccountInfo(userLockupStakingAccount.key);
            if (userLockupStakingAccountInfo) {
                throw new Error("Lockup staking account is already initialized");
            }
            const lockUpIx = yield this.ctx.methods.lockup({
                accounts: {
                    user: this.ctx.wallet.publicKey,
                },
                inputs: {
                    userLockupStakingAccountBump: userLockupStakingAccount.bump,
                    index,
                    vaultIndex,
                    amount,
                },
            });
            if (!lockUpIx.ix) {
                throw new Error("Failed to create locking up instruction");
            }
            tx.add((_a = lockUpIx.ix) === null || _a === void 0 ? void 0 : _a.instructions[0]);
            return tx;
        });
    }
    unlock(index, vaultIndex) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const tx = new web3_js_1.Transaction();
            const unlockIx = yield this.ctx.methods.unlock({
                accounts: {
                    user: this.ctx.wallet.publicKey,
                },
                inputs: {
                    index,
                    vaultIndex,
                },
            });
            if (!unlockIx.ix) {
                throw new Error("Failed to create unlocking instruction");
            }
            tx.add((_a = unlockIx.ix) === null || _a === void 0 ? void 0 : _a.instructions[0]);
            return tx;
        });
    }
}
exports.StakingClient = StakingClient;
