import { Instruction } from "@orca-so/common-sdk";
import { PublicKey } from "@solana/web3.js";
import { Program } from "@coral-xyz/anchor";
import { Staking } from "../artifacts/staking";
import BN from "bn.js";

export type LockupParams = {
  accounts: {
    user: PublicKey;
  };
  inputs: {
    userLockupStakingAccountBump: number;
    index: number;
    vaultIndex: number;
    amount: BN;
  };
};

export async function lockup(
  program: Program<Staking>,
  params: LockupParams,
): Promise<Instruction> {
  const { accounts, inputs } = params;

  const ix = await program.methods
    .lock(
      inputs.userLockupStakingAccountBump,
      inputs.index,
      inputs.vaultIndex,
      inputs.amount,
    )
    .accounts({
      ...accounts,
    })
    .instruction();

  return {
    instructions: [ix],
    cleanupInstructions: [],
    signers: [],
  };
}
