import { Instruction } from "@orca-so/common-sdk";
import { PublicKey } from "@solana/web3.js";
import { Program } from "@coral-xyz/anchor";
import { Staking } from "../artifacts/staking";
import BN from "bn.js";

export type StakeParams = {
  accounts: {
    user: PublicKey;
  };
  inputs: {
    amount: BN;
  };
};

export async function stake(
  program: Program<Staking>,
  params: StakeParams,
): Promise<Instruction> {
  const { accounts, inputs } = params;

  const ix = await program.methods
    .stake(inputs.amount)
    .accounts({
      ...accounts,
    })
    .instruction();

  return {
    instructions: [ix],
    cleanupInstructions: [],
    signers: [],
  };
}
