import { TransactionBuilder, Instruction } from "@orca-so/common-sdk";
import { Context } from "./context";
import * as ixs from "./instructions";

export class Methods {
  public ctx: Context;
  public ix: Instruction | null | undefined;

  public constructor(ctx: Context, ix?: Instruction) {
    this.ctx = ctx;
    this.ix = ix;
  }

  public async initFlexibleAccount(params: ixs.InitFlexibleAccountParams) {
    this.ix = await ixs.initFlexibleAccount(this.ctx.program, params);
    return this;
  }

  public async stake(params: ixs.StakeParams) {
    this.ix = await ixs.stake(this.ctx.program, params);
    return this;
  }

  public async withdraw(params: ixs.WithdrawParams) {
    this.ix = await ixs.withdraw(this.ctx.program, params);
    return this;
  }

  public async withdrawDelay(params: ixs.WithdrawDelayParams) {
    this.ix = await ixs.withdrawDelay(this.ctx.program, params);
    return this;
  }

  public async claim(params: ixs.ClaimParams) {
    this.ix = await ixs.claim(this.ctx.program, params);
    return this;
  }

  public async lockup(params: ixs.LockupParams) {
    this.ix = await ixs.lockup(this.ctx.program, params);
    return this;
  }

  public async unlock(params: ixs.UnlockParams) {
    this.ix = await ixs.unlock(this.ctx.program, params);
    return this;
  }

  public toTx(): TransactionBuilder {
    const tx = new TransactionBuilder(this.ctx.provider.connection, this.ctx.provider.wallet);
    return this.ix ? tx.addInstruction(this.ix) : tx;
  }
}
